import { Component, OnInit, Output, EventEmitter, Inject } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { ToastrService } from 'ngx-toastr';
import { DataService } from '../../../services/data.service';

@Component({
  selector: 'app-create-organizational-unit',
  templateUrl: './create-organizational-unit.component.html',
  styleUrls: ['./create-organizational-unit.component.css']
})
export class CreateOrganizationalUnitComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  classroomInfoFormGroup: FormGroup;
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  typeTranslations: { [key: string]: string } = {
    'organizational-unit': 'Centro',
    'classrooms-group': 'Grupo de aulas',
    'classroom': 'Aula',
    'clients-group': 'Grupo de clientes'
  };
  protected p2pModeOptions = [
    { name: 'Leecher', value: 'leecher' },
    { name: 'Peer', value: 'peer' },
    { name: 'Seeder', value: 'seeder' },
  ];
  protected multicastModeOptions = [
    {"name": 'Half duplex', "value": "half"},
    {"name": 'Full duplex', "value": "full"},
  ];
  parentUnits: any[] = [];
  hardwareProfiles: any[] = [];
  calendars: any[] = [];
  ogLives: any[] = [];
  repositories: any[] = [];
  selectedCalendarUuid: string | null = null;
  parentUnitsWithPaths: { id: string, name: string, path: string }[] = [];

  @Output() unitAdded = new EventEmitter<{ uuid: string; name: string }>();

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient,
    private toastService: ToastrService,
    private dataService: DataService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
    this.generalFormGroup = this._formBuilder.group({
      name: ['', Validators.required],
      parent: [this.data.parent ? this.data.parent['@id'] : null],
      description: [''],
      type: ['', Validators.required]
    });
    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [''],
    });
    this.networkSettingsFormGroup = this._formBuilder.group({
      ogLive: [null],
      ogRepository: [null],
      nextServer: [''],
      bootFileName: [''],
      proxy: [''],
      dns: [''],
      netmask: [''],
      router: [''],
      ntp: [''],
      p2pMode: [''],
      p2pTime: [0, Validators.min(0)],
      mcastIp: [''],
      mcastSpeed: [0, Validators.min(0)],
      mcastPort: [0, Validators.min(0)],
      mcastMode: [''],
      menu: [''],
      hardwareProfile: [''],
      validation: [false]
    });
    this.classroomInfoFormGroup = this._formBuilder.group({
      location: [''],
      projector: [false],
      board: [false],
      capacity: [0, Validators.min(0)],
      remoteCalendar: ['']
    });
  }

  ngOnInit() {
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadCalendars();
    this.loadOgLives();
    this.loadRepositories();
  }

  get filteredTypes(): string[] {
    return this.generalFormGroup.get('parent')?.value ? this.types.filter(type => type !== 'organizational-unit') : this.types;
  }

  loadParentUnits() {
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=1000`;
    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
        this.parentUnitsWithPaths = this.parentUnits.map(unit => ({
          id: unit['@id'],
          name: unit.name,
          path: this.dataService.getOrganizationalUnitPath(unit, this.parentUnits)
        }));
      },
      error => console.error('Error fetching parent units:', error)
    );
  }

  getSelectedParentName(): string | undefined {
    const parentId = this.generalFormGroup.get('parent')?.value;
    return this.parentUnitsWithPaths.find(unit => unit.id === parentId)?.name;
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => this.hardwareProfiles = data,
      error => console.error('Error fetching hardware profiles', error)
    );
  }

  loadOgLives() {
    this.dataService.getOgLives().subscribe(
      (data: any[]) => {
        this.ogLives = data
      },
      error => console.error('Error fetching ogLives', error)
    );
  }

  loadRepositories() {
    this.dataService.getRepositories().subscribe(
      (data: any[]) => this.repositories = data,
      error => console.error('Error fetching repositories', error)
    );
  }

  loadCalendars() {
    const apiUrl = `${this.baseUrl}/remote-calendars?page=1&itemsPerPage=30`;
    this.http.get<any>(apiUrl).subscribe(
      response => this.calendars = response['hydra:member'],
      error => {
        console.error('Error loading calendars', error);
        this.openSnackBar(true, 'Error loading calendars');
      }
    );
  }

  private cleanFormValues(formGroup: FormGroup): any {
    const cleanedValues: any = {};
    Object.keys(formGroup.controls).forEach(key => {
      const value = formGroup.get(key)?.value;
      if (value !== '' && value !== 0) {
        cleanedValues[key] = value;
      }
    });
    return cleanedValues;
  }

  onSubmit() {
    if (this.isFormValid()) {
      const formData: any = this.buildPayload();
      const postUrl = `${this.baseUrl}/organizational-units`;
      const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

      this.http.post<any>(postUrl, formData, { headers }).subscribe(
        response => {
          this.unitAdded.emit(response);
          this.dialogRef.close(response);
          this.toastService.success('Unidad creada exitosamente', 'Éxito');
          this.openSnackBar(false, 'Unidad creada exitosamente');
        },
        error => {
          console.error('Error al realizar POST:', error);
          this.toastService.error('Ha ocurrido un error');
          this.openSnackBar(true, 'Error al crear la unidad organizativa: ' + error.error['hydra:description']);
        }
      );
    }
  }

  private isFormValid(): boolean {
    return this.generalFormGroup.valid &&
           this.additionalInfoFormGroup.valid &&
           this.networkSettingsFormGroup.valid &&
           (this.generalFormGroup.value.type !== 'classroom' || this.classroomInfoFormGroup.valid);
  }

  private buildPayload(): any {
    const generalFormValues = this.cleanFormValues(this.generalFormGroup);
    const additionalInfoFormValues = this.cleanFormValues(this.additionalInfoFormGroup);
    const networkSettingsFormValues = this.cleanFormValues(this.networkSettingsFormGroup);
    const classroomInfoFormValues = this.cleanFormValues(this.classroomInfoFormGroup);

    return {
      ...generalFormValues,
      ...classroomInfoFormValues,
      comments: additionalInfoFormValues.comments,
      networkSettings: { ...networkSettingsFormValues },
      menu: networkSettingsFormValues.menu || null,
      ogLive: networkSettingsFormValues.ogLive || null,
      ogRepository: networkSettingsFormValues.ogRepository || null,
      hardwareProfile: networkSettingsFormValues.hardwareProfile || null,
    };
  }

  onCalendarChange(event: any) {
    this.generalFormGroup.value.remoteCalendar = event.value;
    this.selectedCalendarUuid = event.value;
  }

  onOgLiveChange(event: any) {
    this.networkSettingsFormGroup.value.ogLive = event.value;
  }

  onRepositoryChange(event: any) {
    this.networkSettingsFormGroup.value.ogRepository = event
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  openSnackBar(isError: boolean, message: string) {
    if (isError) {
      this.toastService.error('Error al crear la unidad: ' + message, 'Error');
    } else {
      this.toastService.success('Unidad creada exitosamente', 'Éxito');
    }
  }
}
